﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Text;
using System.Web.UI;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;
using System.Drawing;

namespace Post5675
{
    public partial class _Default : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#B3C6D4"));
                Map1.CurrentExtent = new RectangleShape(-131.22, 55.05, -54.03, 16.91);
                Map1.MapUnit = GeographyUnit.DecimalDegree;

                ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(MapPath("~/app_data/states.shp"));
                worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.FromArgb(255, 243, 239, 228), GeoColor.FromArgb(255, 218, 193, 163), 1);
                worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                LayerOverlay layerOverlay = new LayerOverlay("StaticOverlay");
                layerOverlay.Layers.Add(worldLayer);

                LayerOverlay dynamicOverlay = new LayerOverlay("DynamicOverlay", false, TileType.SingleTile);
                
                InMemoryFeatureLayer shapeLayer = new InMemoryFeatureLayer();
                shapeLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = PointStyles.CreateSimplePointStyle(PointSymbolType.Circle, GeoColor.FromArgb(80, GeoColor.FromHtml("#0033ff")), GeoColor.FromArgb(160, GeoColor.FromHtml("#0033ff")), 10);
                shapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                InMemoryFeatureLayer markerLayer = new InMemoryFeatureLayer();
                GeoImage markerStyle = new GeoImage(MapPath("marker_blue.gif"));
                markerLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle = new PointStyle(markerStyle);
                markerLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.YOffsetInPixel = -12.5f;
                markerLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                dynamicOverlay.Layers.Add("ShapeLayer", shapeLayer);
                dynamicOverlay.Layers.Add("MarkerLayer", markerLayer);

                SimpleMarkerOverlay simpleMarkers = new SimpleMarkerOverlay("FlagMarkers");

                Map1.CustomOverlays.Add(layerOverlay);
                Map1.CustomOverlays.Add(dynamicOverlay);
                Map1.CustomOverlays.Add(simpleMarkers);
            }
        }

        protected void AddMarkers(object sender, EventArgs args)
        {
            string markersInfo = _MarkerInfo.Value;
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(typeof(Collection<JsonMarker>));
            byte[] buffer = Encoding.Default.GetBytes(markersInfo);
            Collection<JsonMarker> jsonMarkers = (Collection<JsonMarker>)serializer.ReadObject(new MemoryStream(buffer));

            LayerOverlay dynamicOverlay = (LayerOverlay)Map1.CustomOverlays["DynamicOverlay"];
            InMemoryFeatureLayer shapeLayer = (InMemoryFeatureLayer)dynamicOverlay.Layers["ShapeLayer"];
            InMemoryFeatureLayer markerLayer = (InMemoryFeatureLayer)dynamicOverlay.Layers["MarkerLayer"];

            foreach (Feature feature in Map1.EditOverlay.Features)
            {
                shapeLayer.InternalFeatures.Add(feature);
            }

            foreach (JsonMarker jsonMarker in jsonMarkers)
            {
                markerLayer.InternalFeatures.Add(new Feature(new PointShape(jsonMarker.X, jsonMarker.Y)));
            }
            Map1.EditOverlay.Features.Clear();
            dynamicOverlay.Redraw();
        }

        protected void SaveClick(object sender, EventArgs args)
        {
            Bitmap bmp = Map1.GetBitmap();
            bmp.Save("c:\\demo.jpeg");
        }
    }

    [DataContract()]
    public class JsonMarker
    {
        [DataMember]
        public string Id { get; set; }
        [DataMember]
        public double Width { get; set; }
        [DataMember]
        public double Height { get; set; }
        [DataMember]
        public double OffsetX { get; set; }
        [DataMember]
        public double OffsetY { get; set; }
        [DataMember]
        public string Url { get; set; }
        [DataMember]
        public double X { get; set; }
        [DataMember]
        public double Y { get; set; }

        public JsonMarker()
        { 
        }

        public Marker ToMarker()
        {
            Marker marker = new Marker(X, Y);
            marker.WebImage.ImageWidth = (int)Width;
            marker.WebImage.ImageHeight = (int)Height;
            marker.WebImage.ImageOffsetX = (float)OffsetX;
            marker.WebImage.ImageOffsetY = (float)OffsetY;
            marker.WebImage.ImageVirtualPath = Url;

            return marker;
        }
    }
}
